==> musashi-core-1.0.3.tar.gz: musashi-core-1.0.3/cmd/txt2xt.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : APIб(2003/06)                                             */
/*============================================================================*/

#include <musashi.h>
#include <stdlib.h>
#include <string.h>

#include <txt2xtHelp.h>
struct mssComHelp comHelp={
  "txt2xt",       /* ޥ̾       */
  "1.0",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

extern struct mssGlobalVariables mssGV;

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* ܤ̾Ȥߤʤե饰                                               */
/*----------------------------------------------------------------------------*/
  MssOptFLG optHFD={
    OFLG,   /* ץ󥿥                                             */
    "F",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    HFDT,   /* ΥץΥȥ(Helpɽ)                         */
    HFDC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̾                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptSLS optFNM={
    OSLS,   /* ץ󥿥                                             */
    "a",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե(ʸ)                                           */
    MssFieldMaxCnt, /* ޤǶڤǿκ                     */
    1,      /* ǤʸĹκǾ                                     */
    MssFieldNameMaxLen,/* ǤʸĹκ                          */
    0,      /* 1:Ǥ˥Ǥ,0:Բ  ex) aaaa:xxxxx            */
    FNMT,   /* ΥץΥȥ(Helpɽ)                         */
    FNMC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ȥ                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptSTR optTIT={
    OSTR,   /* ץ󥿥                                             */
    "l",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե                                                   */
    0,      /* ʸκǾĹ                                               */
    MssTitleMaxLen,    /* ʸκĹ                                    */
    TITT,   /* ΥץΥȥ(Helpɽ)                         */
    TITC    /* ΥץΥ(Helpɽ)                         */
  };  

/*----------------------------------------------------------------------------*/
/*                                                                    */
/*----------------------------------------------------------------------------*/
  MssOptSTR optCOM={
    OSTR,   /* ץ󥿥                                             */
    "c",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե                                                   */
    0,      /* ʸκǾĹ                                               */
    MssCommentMaxLen,/* ʸκĹ                                      */
    COMT,   /* ΥץΥȥ(Helpɽ)                         */
    COMC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ݡȥե                                                         */
/*----------------------------------------------------------------------------*/
  MssOptINF optIMP={
    OINF,   /* ץ󥿥                                             */
    "I",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    IMPT,   /* ΥץΥȥ(Helpɽ)                         */
    IMPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptOTF optOTF={
    OOTF,   /* ץ󥿥                                             */
    "o",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    OTFT,   /* ΥץΥȥ(Helpɽ)                         */
    OTFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optHFD,&optFNM,&optTIT,&optCOM,&optIMP,
               &optINF,&optOTF,&optZIP,NULL};

  struct mssHeader *hdi; /*ƥȥե<head>Ǽ¤*/
  struct mssHeader *hdI; /*-Iϥե<head>Ǽ¤*/
  struct mssHeader *hdo; /*ϥե<head>Ǽ¤*/
  struct mssFPR    *fpr; /*ϥե빽¤*/
  struct mssFPR    *fpr2;/*ݡȥե빽¤*/
  struct mssFPW    *fpw; /*ϥե빽¤*/
  struct mssFldRec *fr;  /*-ԥХåե¤*/

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);        /* ʥʤɤν     */
  mssHelpDoc(opt,&comHelp,argc,argv); /* إ                       */
  mssSetOption(opt,argc,argv);        /* ޥɥץ     */
  fpr=mssOpenFPR(optINF.str,4);       /* ϥե륪ץ         */

/*prnOption(opt);*/
/*prnHeader(hdi);*/

  /*-I,-F,-a,ꤵʤäȤ*/
  if(!optIMP.set && !optHFD.set && !optFNM.set){
    mssShowErrMsg("choose one option in -a,-F,-I");
    mssEnd(mssErrorNoDefault);
  }

  /*-I,-F,-a,ǣİʾꤵ줿*/
  if(optIMP.set+optHFD.set+optFNM.set > 1){
    mssShowErrMsg("choose one option in -a,-F,-I");
    mssEnd(mssErrorNoDefault);
  }
    
  mssGV.txtFlg=1;            /* Ūon*/
  hdi=mssReadHeader(fpr);    /* إåɤ߹             */
  mssGV.txtFlg=0;            /* off*/

/*----------------------------------------------------------------------------*/
/*ϥإåκȽ                                                    */
/*----------------------------------------------------------------------------*/
  fr=mssInitFldRec(hdi->flds->cnt);

  if(optIMP.set){ /*-Iե饰on*/
    fpr2=mssOpenFPR(optIMP.str,4);             /*ݡȥե륪ץ*/
    hdI=mssReadHeader(fpr2);                   /*̾ɤ߹*/
    hdo=mssInitCpyHeader(hdI);                 /*ϥإå*/
    mssAddFieldsByFields(hdo->flds,hdI->flds); /*hdIإåιܾ򥳥ԡ*/
    mssCloseFPR(fpr2);                         /*ݡȥե륯*/
  }else{
    /*ϥإå*/
    hdo=mssInitHeader(optTIT.str,optCOM.str);

    /*ܾ*/

    /*-Fե饰on*/
    if(optHFD.set){
      if( EOF == mssReadFldRec(fpr,fr) ){
        mssShowErrMsg("can not read input file");
        mssEnd(mssErrorNoDefault);
      }
      mssAddFieldsByStrList(hdo->flds, fr->pnt,fr->fldCnt);
    }

    /*-aꤵ줿Ȥ*/
    if(optFNM.set){
      mssAddFieldsByStrList(hdo->flds, optFNM.strList, optFNM.cnt);
    }
  }

  /*ɸϥץ+إåν*/
  fpw=mssOpenFPW(optOTF.str,optZIP.set,0);
  mssWriteHeader(hdo,fpw);

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/

  while( EOF != mssReadFldRec(fpr,fr) ){
    mssGV.inCnt++;
    if(mssGV.inCnt==1){
      if(fr->fldCnt!=hdo->flds->cnt){
      mssShowErrMsg("detected different number of fields in line %d(header has %d, but a record has %d)",mssGV.inCnt,hdo->flds->cnt,fr->fldCnt);
      mssEnd(mssErrorNoDefault);
      }
    }
    mssWriteFld(fr->pnt, fr->fldCnt, "\n", fpw);
    mssGV.outCnt++;
  }
  mssFreeFldRec(fr);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssWriteFooter(fpw);    /*եåν*/
  mssCloseFPR(fpr);       /*ϥեΥ*/
  mssCloseFPW(fpw);       /*ϥեΥ*/
  mssFreeHeader(hdi);     /*ϥإåΰ賫*/
  mssFreeHeader(hdo);     /*ϥإåΰ賫*/
  mssFreeOption(opt);     /*ץΰ賫*/
  mssShowEndMsg();        /*λå*/
  mssEnd(mssExitSuccess); /*λ*/
  return(0); /* to avoid warning message*/
}

==> musashi-core-1.0.4-pre6.tar.gz: musashi-core-1.0.4-pre6/cmd/txt2xt.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : APIб(2003/06)                                             */
/* 1.1 : ܿۤäƤƤ⶯ϥץ(-f)ɲ(2004/07/30)           */
/*     : csv2xtƱͤˤ                                           */
/*============================================================================*/
#include <musashi.h>
#include <stdlib.h>
#include <string.h>

#include <txt2xtHelp.h>
struct mssComHelp comHelp={
  "txt2xt",       /* ޥ̾       */
  "1.1",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

extern struct mssGlobalVariables mssGV;

void setTXTfields(char **fldLst, char *string, int *cnt){
  char *str;
  char *pos;

  (*cnt)=0;
  str=string;

  pos=*(fldLst+*cnt);
  while(1){

    /*ܽüå*/
    if( *str=='\0' || *str==MssFieldDelim ){
      *pos='\0';
      (*cnt)++;
      if(*str=='\0') break;
      pos=*(fldLst+*cnt);
      str++;
      continue;
    }

   /*ʸԡ*/
   *pos++=*str++;
  }
}

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* ܤ̾Ȥߤʤե饰                                               */
/*----------------------------------------------------------------------------*/
  MssOptFLG optHFD={
    OFLG,   /* ץ󥿥                                             */
    "F",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    HFDT,   /* ΥץΥȥ(Helpɽ)                         */
    HFDC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̾                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptSLS optFNM={
    OSLS,   /* ץ󥿥                                             */
    "a",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե(ʸ)                                           */
    MssFieldMaxCnt, /* ޤǶڤǿκ                     */
    1,      /* ǤʸĹκǾ                                     */
    MssFieldNameMaxLen,/* ǤʸĹκ                          */
    0,      /* 1:Ǥ˥Ǥ,0:Բ  ex) aaaa:xxxxx            */
    FNMT,   /* ΥץΥȥ(Helpɽ)                         */
    FNMC    /* ΥץΥ(Helpɽ)                         */
  }; 

/*----------------------------------------------------------------------------*/
/* ȥ                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptSTR optTIT={
    OSTR,   /* ץ󥿥                                             */
    "l",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե                                                   */
    0,      /* ʸκǾĹ                                               */
    MssTitleMaxLen,    /* ʸκĹ                                    */
    TITT,   /* ΥץΥȥ(Helpɽ)                         */
    TITC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/*                                                                    */
/*----------------------------------------------------------------------------*/
  MssOptSTR optCOM={
    OSTR,   /* ץ󥿥                                             */
    "c",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL,   /* ǥե                                                   */
    0,      /* ʸκǾĹ                                               */
    MssCommentMaxLen,/* ʸκĹ                                      */
    COMT,   /* ΥץΥȥ(Helpɽ)                         */
    COMC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ܿۤäƤƤإåˤ碌ƽ                                 */
/*----------------------------------------------------------------------------*/
  MssOptFLG optFRC={
    OFLG,   /* ץ󥿥                                             */
    "f",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    FRCT,   /* ΥץΥȥ(Helpɽ)                         */
    FRCC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ݡȥե                                                         */
/*----------------------------------------------------------------------------*/
  MssOptINF optIMP={
    OINF,   /* ץ󥿥                                             */
    "I",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    IMPT,   /* ΥץΥȥ(Helpɽ)                         */
    IMPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptOTF optOTF={
    OOTF,   /* ץ󥿥                                             */
    "o",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    OTFT,   /* ΥץΥȥ(Helpɽ)                         */
    OTFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optHFD,&optFNM,&optTIT,&optCOM,&optIMP,&optFRC,
               &optINF,&optOTF,&optZIP,NULL};

/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader *hdi; /*ƥȥե<head>Ǽ¤*/
  struct mssHeader *hdI; /*-Iϥե<head>Ǽ¤*/
  struct mssHeader *hdo; /*ϥե<head>Ǽ¤*/
  struct mssFPR    *fpr; /*ϥե빽¤*/
  struct mssFPR    *fpr2;/*ݡȥե빽¤*/
  struct mssFPW    *fpw; /*ϥե빽¤*/
  struct mssFldRec *fr;  /*-ԥХåե¤*/
  struct mssRec    *rec; /*ԥХåե¤*/

  char         **fldLst;
  int            fldCnt;
  int            i,j;
  int            overFlow;
  int            underFlow;
/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);        /* ʥʤɤν     */
  mssHelpDoc(opt,&comHelp,argc,argv); /* إ                       */
  mssSetOption(opt,argc,argv);        /* ޥɥץ     */
  fpr=mssOpenFPR(optINF.str,4);       /* ϥե륪ץ         */

/*prnOption(opt);*/
/*prnHeader(hdi);*/

  /*-I,-F,-a,ꤵʤäȤ*/
  if(!optIMP.set && !optHFD.set && !optFNM.set){
    mssShowErrMsg("choose one option in -a,-F,-I");
    mssEnd(mssErrorNoDefault);
  }

  /*-I,-F,-a,ǣİʾꤵ줿*/
  if(optIMP.set+optHFD.set+optFNM.set > 1){
    mssShowErrMsg("choose one option in -a,-F,-I");
    mssEnd(mssErrorNoDefault);
  }
    
  mssGV.txtFlg=1;            /* Ūon*/
  hdi=mssReadHeader(fpr);    /* إåɤ߹             */
  mssGV.txtFlg=0;            /* off*/

/*----------------------------------------------------------------------------*/
/*ϥإåκȽ                                                    */
/*----------------------------------------------------------------------------*/
  fr=mssInitFldRec(hdi->flds->cnt);
  rec=mssInitRec();

  fldLst=mssMalloc(sizeof(char *)*MssFieldMaxCnt,"txt2xt");
  for(i=0; i<MssFieldMaxCnt; i++){
    *(fldLst+i)=mssMalloc(sizeof(char *)*MssFieldMaxLen,"txt2xt");
  }

  if(optIMP.set){ /*-Iե饰on (إåΥݡ)*/
    fpr2=mssOpenFPR(optIMP.str,4);             /*ݡȥե륪ץ*/
    hdI=mssReadHeader(fpr2);                   /*̾ɤ߹*/
    hdo=mssInitCpyHeader(hdI);                 /*ϥإå*/
    mssAddFieldsByFields(hdo->flds,hdI->flds); /*hdIإåιܾ򥳥ԡ*/
    mssCloseFPR(fpr2);                         /*ݡȥե륯*/
  }else{
    /*ϥإå*/
    hdo=mssInitHeader(optTIT.str,optCOM.str);

    /*ܾ*/

    /*-Fե饰on*/
    if(optHFD.set){
      if( EOF == mssReadRec(fpr,rec) ){
        mssShowErrMsg("can not read input file");
        mssEnd(mssErrorNoDefault);
      }
      setTXTfields(fldLst,rec->pnt,&fldCnt);

      mssAddFieldsByStrList(hdo->flds, fldLst, fldCnt);
    }

    /*-aꤵ줿Ȥ*/
    if(optFNM.set){
      mssAddFieldsByStrList(hdo->flds, optFNM.strList, optFNM.cnt);
    }
  }

  /* ̾NULLå */
  for(i=0; i<hdo->flds->cnt; i++){
    if( *(*(hdo->flds->fi+i))->name == '\0' ){
        mssShowErrMsg("detected NULL field name (%dth field)",i+1);
        mssEnd(mssErrorNoDefault);
    }
  }

  /* Ʊ̾Υå */
  for(i=0; i<hdo->flds->cnt-1; i++){
    for(j=i+1; j<hdo->flds->cnt; j++){
      if(0==strcmp( (*(hdo->flds->fi+i))->name,(*(hdo->flds->fi+j))->name)){
        mssShowErrMsg("detected same field name (%dth and %dth fields)",i+1,j+1);
        mssEnd(mssErrorNoDefault);
      }
    }
  }

  /* ɸϥץ+إåν */
  fpw=mssOpenFPW(optOTF.str,optZIP.set,0);
  mssWriteHeader(hdo,fpw);

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/

  while( EOF != mssReadRec(fpr,rec)){
    mssGV.inCnt++;

    setTXTfields(fldLst,rec->pnt,&fldCnt);

    overFlow=0;
    underFlow=0;
    if(fldCnt!=hdo->flds->cnt) {
      if(optFRC.set){ /*ܿۤäƤƤWARNINGϤΤ*/
        mssShowMsg("detected different number of fields in line %d(header has %d, but the record has %d), adjusted",mssGV.inCnt,hdo->flds->cnt,fldCnt);

        if(fldCnt>hdo->flds->cnt) {
          overFlow=1;
          fldCnt=hdo->flds->cnt;
        }else{
          underFlow=hdo->flds->cnt - fldCnt;
        }
      }else{         /*λ*/
        mssShowErrMsg("detected different number of fields in line %d(header has %d, but the record has %d)",mssGV.inCnt,hdo->flds->cnt,fldCnt);
        mssEnd(mssErrorNoDefault);
      }
    }

    if(underFlow==0){
      for(i=0; i<fldCnt; i++){
        if(i!=0) mssWriteDlm(fpw);
        if(**(fldLst+i)=='\0'){
          mssWriteStr("*",fpw);
        }else{
          mssWriteStr(*(fldLst+i),fpw);
        }
      }

    }else{ /*إåܿʤ*/
      for(i=0; i<fldCnt; i++){
        if(i!=0) mssWriteDlm(fpw);
        if(**(fldLst+i)=='\0'){
          mssWriteStr("*",fpw);
        }else{
          mssWriteStr(*(fldLst+i),fpw);
        }
      }
      for(i=0; i<underFlow; i++){
        mssWriteDlm(fpw);
        mssWriteStr("*",fpw);
      }
    }

    mssWriteRet(fpw);
    mssGV.outCnt++;
  }

  mssFreeRec(rec);
  mssFreeFldRec(fr);

  for(i=0; i<MssFieldMaxCnt; i++){
    mssFree(*(fldLst+i));
  }
  mssFree(fldLst);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssWriteFooter(fpw);    /*եåν*/
  mssCloseFPR(fpr);       /*ϥեΥ*/
  mssCloseFPW(fpw);       /*ϥեΥ*/
  mssFreeHeader(hdi);     /*ϥإåΰ賫*/
  mssFreeHeader(hdo);     /*ϥإåΰ賫*/
  mssFreeOption(opt);     /*ץΰ賫*/
  mssShowEndMsg();        /*λå*/
  mssEnd(mssExitSuccess); /*λ*/
  return(0); /* to avoid warning message*/
}
